module Atmos
   
   #
   # == AttributeHashBase
   #
   # Base class for ACL and Metadata objects.  Contains utility methods common to both,
   # as well as implementations of standard ruby Hash functions to make both ACL and
   # Metadata objects appear as hashes while interacting with Atmos at the same time.
   #
   #
   class AttributeHashBase < Hash # :nodoc:
      attr_reader :last_reload_at
      
      @obj              = nil
      @header           = nil
      @last_reload_at   = nil
      @set_action       = nil
      @reload_action    = nil
      @delete_action    = nil
      
      
      def delete_with_atmos(key)
         delete_without_atmos(key)
      end
      alias :delete_without_atmos :delete
      alias :delete :delete_with_atmos


      def clear_with_atmos
         clear_without_atmos
      end
      alias :clear_without_atmos :clear
      alias :clear :clear_with_atmos


      def replace_with_atmos(h)
         validate_input_hash(h)
         replace_without_atmos(h)
      end
      alias :replace_without_atmos :replace
      alias :replace :replace_with_atmos
      
      
      def merge_with_atmos(h)
         validate_input_hash(h)
         merge_without_atmos(h)
      end
      alias :merge_without_atmos :merge
      alias :merge :merge_with_atmos

      
      def merge_with_atmos!(h)
         validate_input_hash(h)
         
         input = h.clone
         input.each do |k,v|
            input[k] = xlate_value_from_object_to_header(v)
         end
         
         response = @obj.request.do(@set_action, :id => @obj.aoid, @header => Atmos::Util.hash2header(input))
         reload(@reload_action, @obj.aoid)
      end
      alias :merge_without_atmos! :merge!
      alias :merge! :merge_with_atmos!
      alias :update :merge!


      def default_with_atmos=
         raise Atmos::Exceptions::NotImplementedExceptions, "Not implemented for Atmos Object properties."
      end
      alias :default_without_atmos= :default=
      alias :default= :default_with_atmos=
      
      
      #
      # Same as self[key] = value
      #
      def store(key, value)
         self[key] = value
      end
      

      #
      # This method is used internally to generate REST requests to the Atmos server.
      #
      # Each ACL or Metadata object knows what HTTP header it is representing
      # in the Atmos REST API we're wrapping.  This method returns that.
      #
      def header_name
         return @header
      end
      
      
      #
      # This method is used internally to generate REST requests to the Atmos server.
      #
      # Returns data as a String in the form of a valid HTTP header Atmos will
      # recognize.
      #
      def header_value
         Atmos::Util.hash2header(self)
      end
      
      
      #
      # This method is used internally to generate REST requests to the Atmos server.
      #
      def to_header
         "#{self.header_name}: #{self.header_value}"
      end
      
      def to_canonicalized_header
         self.to_header.squeeze
      end
         
         
      protected
         def validate_input_hash(h)
            true
         end
         
         def xlate_value_from_header_to_object(value)
            value
         end

         def reload(action, aoid, options = {})
            response = @obj.request.do(action, options.merge({:id => aoid}))
            Atmos::LOG.debug("AttributeHashBase.reload: type: #{@type}; header: #{@header}; value: #{response[@header]}")
            newval = Atmos::Util.header2hash(response[@header])
            Atmos::LOG.debug("AttributeHashBase.reload: newval: #{newval.inspect}")
            if (!newval.nil?)
               newval.each do |key,val|
                  newval[key] = xlate_value_from_header_to_object(val)
               end
               Atmos::LOG.debug("AttributeHashBase.reload: newval: #{newval.inspect}")
               replace_without_atmos(newval)
            end
            Atmos::LOG.debug("AttributeHashBase.reload: self: #{self.inspect}")

            @last_reload_at = Time.now()
         end

   end
   
   
   # == Access Control Lists (ACLs)
   #
   # There are two hashes for access control, available as properties on the object: +user_acl+ and +group_acl+.  
   # The keys are the Atmos usernames and the values are one of :+none+, :+read+, :+write+, :+full+.  The ACLs
   # behave like normal Hash objects.  All operations are executed against the Atmos server immediately.
   #
   # === Defaults
   # By default, when you create an object, the user you gave as a parameter when instantiating Atmos::Store
   # has full permissions on the object  The default group is +other+.  So:
   #
   #    puts obj.user_acl.inspect => {user => :full}
   #    puts obj.group_acl.inspect => {other => :none}
   #
   #
   # === Adding 
   # Adding permissions for a new user is as easy as adding another hash element:
   #
   #    obj.user_acl[newuser] = :read
   #
   #    puts obj.user_acl.inspect => {user => :full, newuser => :read}
   #
   #
   # === Modifying
   # User and group permissions can be modified by modifying the appropriate key value.
   # Keep in mind that you CAN be dumb and give up access to your own objects,
   # even if there is no other user that has access to them.
   #
   #    obj.user_acl[newuser] = :full
   #    puts obj.user_acl.inspect => {user => :full, newuser => :full}
   #
   #    obj.group_acl['other'] = :full
   #    puts obj.group_acl.inspect => {other => :full}
   #
   #
   # === Deleting
   # Remove any permissions for a given user or group, you can either modify existing permissions
   # to :+none+, or you can delete the user/group name from the appropriate hash.  When you do either,
   # the name disappears entirely from the hash.
   #
   #    obj.user_acl.delete(newuser)
   #    puts obj.user_acl.inspect => {user => :full}
   #
   #    obj.user_acl[newuser] = :none
   #    puts obj.user_acl.inspect => {user => :full}
   #
   #
   class ACL < AttributeHashBase
      USER  = 1
      GROUP = 2
      
      #
      # This constructor is only meant for internal use.  To get ACLs on an object:
      #
      #    obj.user_acl => Hash
      #    obj.group_acl => Hash
      #
      def initialize(obj, type)
         raise Atmos::Exceptions::ArgumentException, "The 'obj' parameter cannot be nil." if (obj.nil?)
         raise Atmos::Exceptions::ArgumentException, "The 'obj' parameter must have an id." if (obj.aoid.nil?)
         raise Atmos::Exceptions::ArgumentException, "The 'type' parameter must be Atmos::ACL::USER or Atmos::ACL::GROUP." if (![USER, GROUP].include?(type))
         
         super()
         
         @obj = obj
         @type = type
         
         @header = (@type == USER) ? 'x-emc-useracl' : 'x-emc-groupacl'
         @delete_action = @set_action = (@type == USER) ? :set_user_acl : :set_group_acl
         @reload_action = :list_acl
         
         reload(@reload_action, @obj.aoid)
      end
      
      
      #
      # Adds or modifies permissions for a user or group.  
      # The change is made on the Atmos server immediately.
      # Valid values are :+none+, :+read+, :+write+, :+full+.
      #
      def []=(key,value)
         validate_value(value)
         response = @obj.request.do(@set_action, :id => @obj.aoid, @header => "#{key}=#{xlate_value_from_object_to_header(value)}")
         reload(@reload_action, @obj.aoid)
      end
      

      #
      # Returns +true+ if this ACL object is representing user ACLs.
      #
      def user?
         @type == USER
      end
      
      #
      # Returns +true+ if this ACL object is representing group ACLs.
      #
      def group?
         @type == GROUP
      end
      
      
      #
      # Removes permissions for specified user/group name.  Update is made on the Atmos server immediately.
      #
      def delete(key)
         response = @obj.request.do(@set_action, :id => @obj.aoid, @header => "#{key}=#{xlate_value_from_object_to_header(:none)}")
         self.delete_without_atmos(key)
         reload(@reload_action, @obj.aoid)         
      end

      #
      # Removes all permissions for all groups, or for all users except the one used to instantiate
      # the Atmos::Store connection.
      #
      def clear
         # do a reload to make absolutely sure ACL is up to date
         reload(@reload_action, @obj.aoid)
         
         values = {}
         self.each do |k,v|
            values[k] = xlate_value_from_object_to_header(:none)
         end
         values.delete(@obj.user)
         
         response = @obj.request.do(@set_action, :id => @obj.aoid, @header => Atmos::Util.hash2header(values))
         reload(@reload_action, @obj.aoid)         
      end
 
       
      private
         def validate_input_hash(h)
            msg = nil
            bad_keys = []
            bad_values = []
            good_values = [:none, :read, :write, :full]
            
            h.each do |k,v|
               bad_keys.push(k) if (k.nil? || !k.kind_of?(String))
               bad_values.push(v) if (v.nil? || !good_values.include?(v))
            end
            
            msg = "The input has was bad: " if (!bad_keys.empty? || !bad_values.empty?)
            msg += "bad keys: #{bad_keys.inspect} " if (!bad_keys.empty?)
            msg += "bad values: #{bad_values.inspect}" if (!bad_values.empty?)
            
            raise Atmos::Exceptions::ArgumentException, msg if (!msg.nil?)
         end

         def validate_value(value)
            if (![:none, :read, :write, :full].include?(value))
               raise Atmos::Exceptions::ArgumentException, "Valid permissions values are :none, :read, :write, :full"
            end
         end
         
         def xlate_value_from_header_to_object(value)
            case value
            when 'NONE'
               :none
            when 'READ'
               :read
            when 'WRITE'
               :write
            when 'FULL_CONTROL'
               :full
            else
               raise Atmos::Exceptions::InternalLibraryException, "Permissions type not recognized: #{value}"
            end
         end
         
         def xlate_value_from_object_to_header(value)
            case value
            when :none
               'NONE'
            when :read
               'READ'
            when :write
               'WRITE'
            when :full
               'FULL_CONTROL'
            end
         end
   end


   # == Metadata
   # In all cases, +metadata+ refers to key/value pairs, and is represented 
   # as a ruby Hash attached to an Atmos::Object object.
   #
   # === Standard Metadata
   # In Atmos, +metadata+ with no modifier (e.g. +listable+ or +system+) 
   # refers to metadata on an object that is not indexed.  In other words,
   # objects cannot be referenced by the metadata information.  The only
   # way to get this metadata information is to already have the object
   # the metadata is attached to.
   #
   # ==== Defaults
   # By default, when you create an object, there is no metadata.  So:
   #
   #    obj.metadata => {}
   #
   # ==== Adding 
   # Add metadata in key/value pairs as you would to a hash.  Keys and 
   # values must be ruby Strings.
   #
   #    obj.metadata[key] = value
   #    obj.metadata.store(key, value)
   #    obj.metadata.merge!({key => value})
   #
   # ==== Modifying
   # Modify metadata as you would a hash.  Keys and values must be ruby 
   # Strings.
   #
   #    obj.metadata[pre-existing-key] = new-value
   #    obj.metadata.store(pre-existing-key, new-value)
   #    obj.metadata.merge!({pre-existing-key => new-value})   
   #
   # ==== Deleting
   # Delete metadata as you would a hash.  Keys must be ruby 
   # Strings.
   #
   #    obj.metadata.delete(key)
   #    obj.metadata.clear
   #
   # === Listable Metadata
   # Listable metadata means atmos indexes the object by the key in the
   # key/value metadata pair.  The keys of listable metadata are also known
   # as listable tags.  Currently, an Atmos server can handle about 10k listable
   # tags easily, so use judiciously with very large datasets.
   #
   # ==== Defaults
   # By default, when you create an object, there is no listable metadata.  So:
   #
   #    obj.listable_metadata => {}
   #
   # ==== Adding 
   # Add metadata in key/value pairs as you would to a hash.  Keys and 
   # values must be ruby Strings.
   #
   #    obj.listable_metadata[key] = value
   #    obj.listable_metadata.store(key, value)
   #    obj.listable_metadata.merge!({key => value})
   #
   # ==== Modifying
   # Modify metadata as you would a hash.  Keys and values must be ruby 
   # Strings.
   #
   #    obj.listable_metadata[pre-existing-key] = new-value
   #    obj.listable_metadata.store(pre-existing-key, new-value)
   #    obj.listable_metadata.merge!({pre-existing-key => new-value})   
   #
   # ==== Deleting
   # Delete metadata as you would a hash.  Keys must be ruby 
   # Strings.
   #
   #    obj.listable_metadata.delete(key)
   #    obj.listable_metadata.clear
   #
   # === System Metadata
   # System metadata is a standard group of information maintained by Atmos for each object,
   # such as +atime+, +mtime+, +type+, +policyname+.
   #
   # System metadata is not modifiable.
   #
   class Metadata < AttributeHashBase
      LISTABLE     = 1
      NON_LISTABLE = 2
      SYSTEM       = 3


      #
      # This constructor is only meant for internal use.  To get the metadata on an object:
      #
      #    obj.metadata => Hash
      #    obj.listable_metadata => Hash
      #    obj.system_metadata => Hash
      #
      def initialize(obj, type)
         raise Atmos::Exceptions::ArgumentException, "The 'obj' parameter cannot be nil." if (obj.nil?)
         raise Atmos::Exceptions::ArgumentException, "The 'obj' parameter must have an id." if (obj.aoid.nil?)
         raise Atmos::Exceptions::ArgumentException, "The 'type' parameter must be one of Atmos::Metadata::LISTABLE Atmos::Metadata::NON_LISTABLE Atmos::Metadata::SYSTEM." if (![SYSTEM, LISTABLE, NON_LISTABLE].include?(type))

         super()
         
         @obj = obj
         @type = type
         
         @header = (@type == LISTABLE) ? 'x-emc-listable-meta' : 'x-emc-meta'
         @reload_action = (@type == SYSTEM) ? :list_system_metadata : :list_metadata
         @set_action = (@type == LISTABLE) ? :set_listable_metadata : :set_metadata

         reload(@reload_action, @obj.aoid)
      end
      
      
      #
      # Adds the specified metadata to the object unless the object is representing system metadata.
      #
      # System metadata cannot be modified, so an Atmos::Exceptions::NotImplementedException 
      # is thrown.
      #
      # The change is made on the Atmos server immediately.
      # 
      def []=(key,value)
         raise Atmos::Exceptions::NotImplementedException, "System metadata cannot be modified." if (@type == SYSTEM)
         raise Atmos::Exceptions::ArgumentException, "The 'value' parameter must be of type String'." if (!value.nil? && !value.kind_of?(String))
         response = @obj.request.do(@set_action, :id => @obj.aoid, @header => "#{key}=#{value}")
         reload(@reload_action, @obj.aoid)
      end
      
      
      #
      # Deletes all metadata from the object unless the object is representing system metadata.
      #
      # System metadata cannot be modified, so an Atmos::Exceptions::NotImplementedException 
      # is thrown.
      #
      # The change is made on the Atmos server immediately.
      # 
      def clear
         raise Atmos::Exceptions::NotImplementedException, "System metadata cannot be modified." if (@type == SYSTEM)
         reload(@reload_action, @obj.aoid)         
         response = @obj.request.do(:delete_metadata, :id => @obj.aoid, 'x-emc-tags' => self.keys.join(','))
         self.clear_without_atmos
         reload(@reload_action, @obj.aoid)         
      end
      
      
      #
      # Deletes the specified metadata from the object unless the 
      # object is representing system metadata.  System metadata cannot
      # be modified, so an <tt>Atmos::Exceptions::NotImplementedException</tt>
      # is thrown.
      #
      # The deleted is executed on the Atmos server immediately.
      # 
      # Required:
      #   *<tt>key</tt> - 
      def delete(key)
         raise Atmos::Exceptions::NotImplementedException, "System metadata cannot be modified." if (@type == SYSTEM)
         response = @obj.request.do(:delete_metadata, :id => @obj.aoid, 'x-emc-tags' => key)
         self.delete_without_atmos(key)
         reload(@reload_action, @obj.aoid)         
      end
      
      
      #
      # Returns +true+ if this Metadata object is representing listable metadata.
      #
      def listable?
         @type == LISTABLE
      end
      
      
      #
      # Returns +true+ if this Metadata object is representing non-listable metadata.
      #
      def non_listable?
         @type == NON_LISTABLE
      end
      
      
      #
      # Returns +true+ if this Metadata object is representing system metadata.
      #
      def system?
         @type == SYSTEM
      end
      
      
      private
         #
         # This method is defined to override AttributeHashBase.validate_input_hash,
         # which simply returns true.  This allows the base class to contain the
         # method definitions for several standard Hash functions, yet allows the
         # validation to be specialized for each subclass.
         #
         # Required:
         # *<tt>h</tt> - the hash to validate
         #
         def validate_input_hash(h)
            msg         = nil
            bad_keys    = []
            bad_values  = []
            good_values = [:none, :read, :write, :full]
            
            h.each do |k,v|
               bad_keys.push(k)   if (k.nil? || !k.kind_of?(String))
               bad_values.push(v) if (v.nil? || !good_values.include?(v))
            end
            
            msg = "The input has was bad: " if (!bad_keys.empty? || !bad_values.empty?)
            msg += "bad keys: #{bad_keys.inspect} " if (!bad_keys.empty?)
            msg += "bad values: #{bad_values.inspect}" if (!bad_values.empty?)
            
            raise Atmos::Exceptions::ArgumentException, msg if (!msg.nil?)
         end

   end
   
end
