require 'credentials'
require 'test/unit'
require 'atmos'


class AtmosACLCreateTest < Test::Unit::TestCase
   @cleanup = []

   
   def initialize(id)
      super(id)
      @cleanup = []
   end

   def setup
      @s = Atmos::Test::Util.static_store
      @obj = @s.create()
      @cleanup.push(@obj)
   end

   def teardown
      @cleanup.each do |obj|
         obj.delete
      end
   end
   
   def test_create_acl_nil_obj
      assert_raise Atmos::Exceptions::ArgumentException do
         Atmos::ACL.new(nil, Atmos::ACL::USER)
      end
   end
   
   def test_create_acl_nil_type
      assert_raise Atmos::Exceptions::ArgumentException do
         Atmos::ACL.new(@obj, nil)
      end
   end
   
   def test_create_acl_bad_type
      assert_raise Atmos::Exceptions::ArgumentException do
         Atmos::ACL.new(@obj, 42)
      end
   end
      
   def test_create_acl_invalid_options
      assert_raise Atmos::Exceptions::ArgumentException do
         Atmos::ACL.new(@obj, 'group')
      end

      assert_raise Atmos::Exceptions::ArgumentException do
         Atmos::ACL.new(@obj, 'user')
      end
   end
   
   def test_create_acl_valid_options
      acl = Atmos::ACL.new(@obj, Atmos::ACL::GROUP)
      assert_equal('x-emc-groupacl', acl.header_name)
      assert(!acl.user?)
      assert(acl.group?)

      acl = Atmos::ACL.new(@obj, Atmos::ACL::USER)
      assert_equal('x-emc-useracl', acl.header_name)
      assert(acl.user?)
      assert(!acl.group?)      
   end

end


class AtmosACLUpdateTest < Test::Unit::TestCase
   @cleanup = []
   
   def initialize(id)
      super(id)
      @cleanup = []
   end

   def setup
      @s = Atmos::Store.new(:url    => Atmos::Test::Util.url,
                            :uid    => "#{Atmos::Test::Util.tenant}/#{Atmos::Test::Util.user}",
                            :secret => Atmos::Test::Util.secret)
      @obj = @s.create()
      @cleanup.push(@obj)
   end

   def teardown
      @cleanup.each do |obj|
         obj.delete
      end
      $stdout.flush
   end

   def test_add_user_acl_with_bad_value_symbol
      assert_raise Atmos::Exceptions::ArgumentException do
         @obj.user_acl[Atmos::Test::Util.other_user] = :whatever
      end
   end
   
   def test_add_user_acl_with_bad_value_string
      assert_raise Atmos::Exceptions::ArgumentException do
         @obj.user_acl[Atmos::Test::Util.other_user] = 'read'
      end
   end
   
   def test_add_group_acl_with_bad_value_symbol
      assert_raise Atmos::Exceptions::ArgumentException do
         @obj.user_acl[Atmos::Test::Util.other_group] = :whatever
      end
   end
   
   def test_add_group_acl_with_bad_value_string
      assert_raise Atmos::Exceptions::ArgumentException do
         @obj.user_acl[Atmos::Test::Util.other_group] = 'write'
      end
   end
   
   def test_add_user_acl_with_bad_user
   end
   
   def test_add_group_acl_with_bad_group
   end
   
   def test_add_user_acl_good_values
      @obj.user_acl[Atmos::Test::Util.other_user] = :read
      assert_equal({Atmos::Test::Util.user => :full, Atmos::Test::Util.other_user => :read}, @obj.user_acl)
      assert(@obj.user_acl.has_key?(Atmos::Test::Util.user))
      assert(@obj.user_acl.has_key?(Atmos::Test::Util.other_user))
      assert_equal(:full, @obj.user_acl[Atmos::Test::Util.user])
      assert_equal(:read, @obj.user_acl[Atmos::Test::Util.other_user])
   end
   
   def test_add_group_acl_good_values
   end
   
   def test_change_default_user_permissions
   end
   
   def test_change_default_group_permissions
      assert_equal({Atmos::Test::Util.group => :none}, @obj.group_acl)
      @obj.group_acl[Atmos::Test::Util.group] = :read
      assert_equal({Atmos::Test::Util.group => :read}, @obj.group_acl)
      assert(@obj.group_acl.has_key?(Atmos::Test::Util.group))
      assert_equal(:read, @obj.group_acl[Atmos::Test::Util.group])
   end
   
   def test_change_secondary_user_permissions
      @obj.user_acl[Atmos::Test::Util.other_user] = :read
      assert_equal({Atmos::Test::Util.user => :full, Atmos::Test::Util.other_user => :read}, @obj.user_acl)
      assert(@obj.user_acl.has_key?(Atmos::Test::Util.user))
      assert(@obj.user_acl.has_key?(Atmos::Test::Util.other_user))
      assert_equal(:full, @obj.user_acl[Atmos::Test::Util.user])
      assert_equal(:read, @obj.user_acl[Atmos::Test::Util.other_user])

      @obj.user_acl[Atmos::Test::Util.other_user] = :none
      assert_equal({Atmos::Test::Util.user => :full}, @obj.user_acl)     
      assert(@obj.user_acl.has_key?(Atmos::Test::Util.user))
      assert(!@obj.user_acl.has_key?(Atmos::Test::Util.other_user))
      assert_equal(:full, @obj.user_acl[Atmos::Test::Util.user])

      @obj.user_acl[Atmos::Test::Util.other_user] = :full
      assert_equal({Atmos::Test::Util.user => :full, Atmos::Test::Util.other_user => :full}, @obj.user_acl)     
      assert(@obj.user_acl.has_key?(Atmos::Test::Util.user))
      assert(@obj.user_acl.has_key?(Atmos::Test::Util.other_user))
      assert_equal(:full, @obj.user_acl[Atmos::Test::Util.user])
      assert_equal(:full, @obj.user_acl[Atmos::Test::Util.other_user])
   end
   
   def test_change_secondary_group_permissions
   end
   
   def test_delete_user_via_no_permissions
      @obj.user_acl[Atmos::Test::Util.other_user] = :read
      assert_equal({Atmos::Test::Util.user => :full, Atmos::Test::Util.other_user => :read}, @obj.user_acl)
      assert(@obj.user_acl.has_key?(Atmos::Test::Util.user))
      assert(@obj.user_acl.has_key?(Atmos::Test::Util.other_user))
      assert_equal(:full, @obj.user_acl[Atmos::Test::Util.user])
      assert_equal(:read, @obj.user_acl[Atmos::Test::Util.other_user])
 
       @obj.user_acl[Atmos::Test::Util.other_user] = :none
      assert_equal({Atmos::Test::Util.user => :full}, @obj.user_acl)
      assert(@obj.user_acl.has_key?(Atmos::Test::Util.user))
      assert(!@obj.user_acl.has_key?(Atmos::Test::Util.other_user))
      assert_equal(:full, @obj.user_acl[Atmos::Test::Util.user])
    end
   
   def test_delete_group_via_no_permissions
   end

   def test_delete_user_via_delete
      @obj.user_acl[Atmos::Test::Util.other_user] = :read
      assert_equal({Atmos::Test::Util.user => :full, Atmos::Test::Util.other_user => :read}, @obj.user_acl)
      assert(@obj.user_acl.has_key?(Atmos::Test::Util.user))
      assert(@obj.user_acl.has_key?(Atmos::Test::Util.other_user))
      assert_equal(:full, @obj.user_acl[Atmos::Test::Util.user])
      assert_equal(:read, @obj.user_acl[Atmos::Test::Util.other_user])
 
      @obj.user_acl.delete(Atmos::Test::Util.other_user)
      assert_equal({Atmos::Test::Util.user => :full}, @obj.user_acl)
      assert(@obj.user_acl.has_key?(Atmos::Test::Util.user))
      assert(!@obj.user_acl.has_key?(Atmos::Test::Util.other_user))
      assert_equal(:full, @obj.user_acl[Atmos::Test::Util.user])
   end
   
   def test_delete_group_via_delete
   end
   
   def test_clear_users
      @obj.user_acl[Atmos::Test::Util.other_user] = :read
      assert_equal({Atmos::Test::Util.user => :full, Atmos::Test::Util.other_user => :read}, @obj.user_acl)
      assert(@obj.user_acl.has_key?(Atmos::Test::Util.user))
      assert(@obj.user_acl.has_key?(Atmos::Test::Util.other_user))
      assert_equal(:full, @obj.user_acl[Atmos::Test::Util.user])
      assert_equal(:read, @obj.user_acl[Atmos::Test::Util.other_user])
 
      @obj.user_acl.clear
      assert_equal({Atmos::Test::Util.user => :full}, @obj.user_acl)
      assert(@obj.user_acl.has_key?(Atmos::Test::Util.user))
      assert(!@obj.user_acl.has_key?(Atmos::Test::Util.other_user))
      assert_equal(:full, @obj.user_acl[Atmos::Test::Util.user])
   end
   
   def test_clear_groups
   end
   
   def test_bad_merge_users
      assert_raise Atmos::Exceptions::ArgumentException do
         @obj.user_acl.merge({Atmos::Test::Util.other_user => 'foo'})
      end
   end
   
   def test_merge_users
      assert_equal({Atmos::Test::Util.user => :full}, @obj.user_acl)

      rv = @obj.user_acl.merge({Atmos::Test::Util.other_user => :read})
      assert_equal({Atmos::Test::Util.user => :full, Atmos::Test::Util.other_user => :read}, rv)
      assert_equal({Atmos::Test::Util.user => :full}, @obj.user_acl)
   end
   
   def test_merge_groups
   end
   
   def test_bad_merge_bang_users
      assert_raise Atmos::Exceptions::ArgumentException do
         @obj.user_acl.merge!({Atmos::Test::Util.other_user => 'foo'})
      end
   end
   
   def test_merge_bang_users
      @obj.user_acl.merge!({Atmos::Test::Util.other_user => :read})
      assert_equal({Atmos::Test::Util.user => :full, Atmos::Test::Util.other_user => :read}, @obj.user_acl)
   end
   
   def test_bad_merge_bang_groups
   end
   
   def test_merge_bang_groups
   end
   
   def test_bad_replace_users
      assert_raise Atmos::Exceptions::ArgumentException do
         @obj.user_acl.replace({Atmos::Test::Util.other_user => 'foo'})
      end
   end

   def test_replace_users
      assert_equal({Atmos::Test::Util.user => :full}, @obj.user_acl)

      @obj.user_acl.replace({Atmos::Test::Util.other_user => :read})
      assert_equal({Atmos::Test::Util.other_user => :read}, @obj.user_acl)
   end
   
   def test_replace_groups
   end
   
   
   
#   def test_give_away_access
#      @obj.user_acl[Atmos::Test::Util.user] = :read
#      assert_equal({Atmos::Test::Util.user => :read}, @obj.user_acl)
#   end
   

end
