require 'credentials'
require 'test/unit'
require 'atmos'


class AtmosObjectCreateTest < Test::Unit::TestCase

   def initialize(id)
      super(id)
      @cleanup = []
   end

   def setup
      @s = Atmos::Test::Util.static_store
   end


   def teardown
      @cleanup.each do |obj|
         obj.delete
      end
      $stdout.flush
   end

   def test_create_empty_object_via_object_interface
      obj = @s.create
      @cleanup.push(obj)

      assert_not_nil(obj)
      assert_equal({}, obj.metadata)
      assert_equal({}, obj.listable_metadata)
      assert_equal({Atmos::Test::Util.user => :full}, obj.user_acl)
      assert_equal({Atmos::Test::Util.group => :none}, obj.group_acl)
   end
  
   def test_create_empty_object_via_namespace_interface
      time = Time.now()
      time = time.to_i
      ns   = "/foo/bar#{time}"

      obj = @s.create(:namespace => ns)
      @cleanup.push(obj)

      assert_not_nil(obj)
      assert_equal({}, obj.metadata)
      assert_equal({}, obj.listable_metadata)
      assert_equal({Atmos::Test::Util.user => :full}, obj.user_acl)
      assert_equal({Atmos::Test::Util.group => :none}, obj.group_acl)
   end
  
   def test_create_object_with_bad_option
      assert_raise Atmos::Exceptions::ArgumentException do 
         @cleanup.push(@s.create(:tag => ['testtag']))
      end
   end
  
   def test_create_object_with_md_as_string
      assert_raise Atmos::Exceptions::ArgumentException do 
         @cleanup.push(@s.create(:metadata => 'testtag'))
      end
   end
   
   def test_create_object_with_md_as_array
      assert_raise Atmos::Exceptions::ArgumentException do 
         @cleanup.push(@s.create(:metadata => ['testtag']))
      end
   end
   
   def test_create_object_with_metadata
      md    = {'non' => 'listable', 'user' => 'metadata', 'a' => 'b'}
      obj = @s.create(:metadata => md)
      @cleanup.push(obj)
      assert_not_nil(obj)
      
      assert(obj.metadata.non_listable?)
      assert(!obj.metadata.listable?)
      assert_not_equal({}, obj.metadata)
      assert_equal(md, obj.metadata)
      
      assert_equal({}, obj.listable_metadata)
      assert_equal({Atmos::Test::Util.user => :full}, obj.user_acl)
      assert_equal({Atmos::Test::Util.group => :none}, obj.group_acl)
   end
  
   def test_create_object_with_listable_metadata
      lmd = {'x' => 'a', 'y' => 'b', 'z' => 'c'}
      obj = @s.create(:listable_metadata => lmd)
      @cleanup.push(obj)
      assert_not_nil(obj)
      assert_equal({}, obj.metadata)
      assert_equal({Atmos::Test::Util.user => :full}, obj.user_acl)
      assert_equal({Atmos::Test::Util.group => :none}, obj.group_acl)
      
      assert_not_equal({}, obj.listable_metadata)
      assert_equal(lmd, obj.listable_metadata)
      assert(!obj.listable_metadata.non_listable?)
      assert(obj.listable_metadata.listable?)
   end
  
   def test_create_object_with_mixed_metadata
      md    = {'non' => 'listable', 'user' => 'metadata', 'a' => 'b'}
      lmd = {'x' => 'a', 'y' => 'b', 'z' => 'c'}
      obj = @s.create(:metadata => md, :listable_metadata => lmd)
      @cleanup.push(obj)
      assert_not_nil(obj)
      assert_equal({Atmos::Test::Util.user => :full}, obj.user_acl)
      assert_equal({Atmos::Test::Util.group => :none}, obj.group_acl)
      
      assert(!obj.listable_metadata.non_listable?)
      assert(obj.listable_metadata.listable?)
      assert_not_equal({}, obj.listable_metadata)
      assert_equal(lmd, obj.listable_metadata)

      assert(obj.metadata.non_listable?)
      assert(!obj.metadata.listable?)
      assert_not_equal({}, obj.metadata)
      assert_equal(md, obj.metadata)
   end
  
#  def test_create_object_with_acl
#  end
  
   def test_create_object_with_data_as_string_with_length
      data = "some stupid blob content"
      obj = @s.create(:data => data, :length => data.length)
      @cleanup.push(obj)
      assert_equal(data, obj.data)
   end
   
   def test_create_object_with_data_as_string_without_length
      data = "some stupid blob content"
      obj = @s.create(:data => data)
      @cleanup.push(obj)
      assert_equal(data, obj.data)
   end
   
   def test_create_object_with_data_as_stream_with_length
      contents = open(Atmos::Test::Util.small_text_filename).read
      data = open(Atmos::Test::Util.small_text_filename)
      obj = @s.create(:data => data, :length => data.stat.size)
      @cleanup.push(obj)
      assert_equal(contents, obj.data)
   end

   def test_create_object_with_data_as_stream_without_length
      file = open(Atmos::Test::Util.small_text_filename)
      contents = file.read
      file.close
      data = open(Atmos::Test::Util.small_text_filename)
      obj = @s.create(:data => data)
      @cleanup.push(obj)
      assert_equal(contents, obj.data)
   end
   
   def donttest_create_with_IO_as_data
       @s   = Atmos::Test::Util.static_store
    @md  = {'non' => 'listable', 'user' => 'metadata', 'a' => 'b'}
    @lmd = {'x' => 'a', 'y' => 'b', 'z' => 'c'}
    data = open(Atmos::Test::Util.small_binary_filename)
    obj  = @s.create(:metadata => @md, 
                     :listable_metadata => @lmd,
                     :data => File.new(Atmos::Test::Util.small_binary_filename, "rb"))
    @cleanup.push(obj)
    @aoid = obj.aoid
 end

end
